@@ -16,6 +16,7 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # Copyright (C) 2012 Lanedo GmbH
+# Copyright (C) 2012-2015 Aleksander Morgado <aleksander@aleksander.es>
 #
 
 import string
@@ -43,130 +44,88 @@ class VariableString(Variable):
             # Fixed-size strings
             self.needs_dispose = False
             self.length_prefix_size = 0
+            self.n_size_prefix_bytes = 0
             self.fixed_size = dictionary['fixed-size']
             self.max_size = ''
         else:
             self.is_fixed_size = False
+            self.fixed_size = '-1'
             # Variable-length strings in heap
             self.needs_dispose = True
             if 'size-prefix-format' in dictionary:
                 if dictionary['size-prefix-format'] == 'guint8':
                     self.length_prefix_size = 8
+                    self.n_size_prefix_bytes = 1
                 elif dictionary['size-prefix-format'] == 'guint16':
                     self.length_prefix_size = 16
+                    self.n_size_prefix_bytes = 2
                 else:
                     raise ValueError('Invalid size prefix format (%s): not guint8 or guint16' % dictionary['size-prefix-format'])
             # Strings which are given as the full value of a TLV and which don't have
             # a explicit 'size-prefix-format' will NOT have a length prefix
             elif 'type' in dictionary and dictionary['type'] == 'TLV':
                 self.length_prefix_size = 0
+                self.n_size_prefix_bytes = 0
             else:
                 # Default to UINT8
                 self.length_prefix_size = 8
-            self.fixed_size = ''
+                self.n_size_prefix_bytes = 1
             self.max_size = dictionary['max-size'] if 'max-size' in dictionary else ''
 
 
     """
     Read a string from the raw byte buffer.
     """
-    def emit_buffer_read(self, f, line_prefix, variable_name, buffer_name, buffer_len):
-        translations = { 'lp'             : line_prefix,
-                         'variable_name'  : variable_name,
-                         'buffer_name'    : buffer_name,
-                         'buffer_len'     : buffer_len }
+    def emit_buffer_read(self, f, line_prefix, tlv_out, error, variable_name):
+        translations = { 'lp'            : line_prefix,
+                         'tlv_out'       : tlv_out,
+                         'variable_name' : variable_name,
+                         'error'         : error }
 
         if self.is_fixed_size:
             translations['fixed_size'] = self.fixed_size
-            template = (
-                '${lp}/* Read the fixed-size string variable from the buffer */\n'
-                '${lp}qmi_utils_read_fixed_size_string_from_buffer (\n'
-                '${lp}    &${buffer_name},\n'
-                '${lp}    &${buffer_len},\n'
-                '${lp}    ${fixed_size},\n'
-                '${lp}    &${variable_name}[0]);\n'
-                '${lp}${variable_name}[${fixed_size}] = \'\\0\';\n')
+
+            # Fixed sized strings exposed in public fields will need to be
+            # explicitly allocated in heap
+            if self.public:
+                translations['fixed_size_plus_one'] = int(self.fixed_size) + 1
+                template = (
+                    '${lp}${variable_name} = g_malloc (${fixed_size_plus_one});\n'
+                    '${lp}if (!qmi_message_tlv_read_fixed_size_string (message, init_offset, &offset, ${fixed_size}, &${variable_name}[0], ${error})) {\n'
+                    '${lp}    g_free (${variable_name});\n'
+                    '${lp}    ${variable_name} = NULL;\n'
+                    '${lp}    goto ${tlv_out};\n'
+                    '${lp}}\n'
+                    '${lp}${variable_name}[${fixed_size}] = \'\\0\';\n')
+            else:
+                template = (
+                    '${lp}if (!qmi_message_tlv_read_fixed_size_string (message, init_offset, &offset, ${fixed_size}, &${variable_name}[0], ${error}))\n'
+                    '${lp}    goto ${tlv_out};\n'
+                    '${lp}${variable_name}[${fixed_size}] = \'\\0\';\n')
         else:
-            translations['length_prefix_size'] = self.length_prefix_size
+            translations['n_size_prefix_bytes'] = self.n_size_prefix_bytes
             translations['max_size'] = self.max_size if self.max_size != '' else '0'
             template = (
-                '${lp}/* Read the string variable from the buffer */\n'
-                '${lp}qmi_utils_read_string_from_buffer (\n'
-                '${lp}    &${buffer_name},\n'
-                '${lp}    &${buffer_len},\n'
-                '${lp}    ${length_prefix_size},\n'
-                '${lp}    ${max_size},\n'
-                '${lp}    &(${variable_name}));\n')
-
-        f.write(string.Template(template).substitute(translations))
-
-
-    """
-    Emits the code involved in computing the size of the variable.
-    """
-    def emit_size_read(self, f, line_prefix, variable_name, buffer_name, buffer_len):
-        translations = { 'lp'            : line_prefix,
-                         'variable_name' : variable_name,
-                         'buffer_name'   : buffer_name,
-                         'buffer_len'    : buffer_len }
-
-        if self.is_fixed_size:
-            translations['fixed_size'] = self.fixed_size
-            template = (
-                '${lp}${variable_name} += ${fixed_size};\n')
-        elif self.length_prefix_size == 0:
-            template = (
-                '${lp}${variable_name} += ${buffer_len};\n')
-        elif self.length_prefix_size == 8:
-            template = (
-                '${lp}{\n'
-                '${lp}    guint8 size8;\n'
-                '${lp}    const guint8 *aux_buffer = &${buffer_name}[${variable_name}];\n'
-                '${lp}    guint16 aux_buffer_len = ${buffer_len} - ${variable_name};\n'
-                '\n'
-                '${lp}    qmi_utils_read_guint8_from_buffer (&aux_buffer, &aux_buffer_len, &size8);\n'
-                '${lp}    ${variable_name} += (1 + size8);\n'
-                '${lp}}\n')
-        elif self.length_prefix_size == 16:
-            template = (
-                '${lp}{\n'
-                '${lp}    guint16 size16;\n'
-                '${lp}    const guint8 *aux_buffer = &${buffer_name}[${variable_name}];\n'
-                '${lp}    guint16 aux_buffer_len = ${buffer_len} - ${variable_name};\n'
-                '\n'
-                '${lp}    qmi_utils_read_guint16_from_buffer (&aux_buffer, &aux_buffer_len, QMI_ENDIAN_LITTLE, &size16);\n'
-                '${lp}    ${variable_name} += (2 + size16);\n'
-                '${lp}}\n')
+                '${lp}if (!qmi_message_tlv_read_string (message, init_offset, &offset, ${n_size_prefix_bytes}, ${max_size}, &(${variable_name}), ${error}))\n'
+                '${lp}    goto ${tlv_out};\n')
         f.write(string.Template(template).substitute(translations))
 
 
     """
     Write a string to the raw byte buffer.
     """
-    def emit_buffer_write(self, f, line_prefix, variable_name, buffer_name, buffer_len):
-        translations = { 'lp'             : line_prefix,
-                         'variable_name'  : variable_name,
-                         'buffer_name'    : buffer_name,
-                         'buffer_len'     : buffer_len }
+    def emit_buffer_write(self, f, line_prefix, tlv_name, variable_name):
+        translations = { 'lp'                  : line_prefix,
+                         'tlv_name'            : tlv_name,
+                         'variable_name'       : variable_name,
+                         'fixed_size'          : self.fixed_size,
+                         'n_size_prefix_bytes' : self.n_size_prefix_bytes }
 
-        if self.is_fixed_size:
-            translations['fixed_size'] = self.fixed_size
-            template = (
-                '${lp}/* Write the fixed-size string variable to the buffer */\n'
-                '${lp}qmi_utils_write_fixed_size_string_to_buffer (\n'
-                '${lp}    &${buffer_name},\n'
-                '${lp}    &${buffer_len},\n'
-                '${lp}    ${fixed_size},\n'
-                '${lp}    ${variable_name});\n')
-        else:
-            translations['length_prefix_size'] = self.length_prefix_size
-            template = (
-                '${lp}/* Write the string variable to the buffer */\n'
-                '${lp}qmi_utils_write_string_to_buffer (\n'
-                '${lp}    &${buffer_name},\n'
-                '${lp}    &${buffer_len},\n'
-                '${lp}    ${length_prefix_size},\n'
-                '${lp}    ${variable_name});\n')
+        template = (
+            '${lp}if (!qmi_message_tlv_write_string (self, ${n_size_prefix_bytes}, ${variable_name}, ${fixed_size}, error)) {\n'
+            '${lp}    g_prefix_error (error, "Cannot write string in TLV \'${tlv_name}\': ");\n'
+            '${lp}    goto error_out;\n'
+            '${lp}}\n')
 
         f.write(string.Template(template).substitute(translations))
 
@@ -174,11 +133,8 @@ class VariableString(Variable):
     """
     Get the string as printable
     """
-    def emit_get_printable(self, f, line_prefix, printable, buffer_name, buffer_len):
-        translations = { 'lp'             : line_prefix,
-                         'printable'      : printable,
-                         'buffer_name'    : buffer_name,
-                         'buffer_len'     : buffer_len }
+    def emit_get_printable(self, f, line_prefix):
+        translations = { 'lp' : line_prefix }
 
         if self.is_fixed_size:
             translations['fixed_size'] = self.fixed_size
@@ -188,33 +144,22 @@ class VariableString(Variable):
                 '${lp}{\n'
                 '${lp}    gchar tmp[${fixed_size_plus_one}];\n'
                 '\n'
-                '${lp}    /* Read the fixed-size string variable from the buffer */\n'
-                '${lp}    qmi_utils_read_fixed_size_string_from_buffer (\n'
-                '${lp}        &${buffer_name},\n'
-                '${lp}        &${buffer_len},\n'
-                '${lp}        ${fixed_size},\n'
-                '${lp}        &tmp[0]);\n'
+                '${lp}    if (!qmi_message_tlv_read_fixed_size_string (message, init_offset, &offset, ${fixed_size}, &tmp[0], &error))\n'
+                '${lp}        goto out;\n'
                 '${lp}    tmp[${fixed_size}] = \'\\0\';\n'
-                '\n'
-                '${lp}    g_string_append_printf (${printable}, "%s", tmp);\n'
+                '${lp}    g_string_append (printable, tmp);\n'
                 '${lp}}\n')
         else:
-            translations['length_prefix_size'] = self.length_prefix_size
+            translations['n_size_prefix_bytes'] = self.n_size_prefix_bytes
             translations['max_size'] = self.max_size if self.max_size != '' else '0'
             template = (
                 '\n'
                 '${lp}{\n'
                 '${lp}    gchar *tmp;\n'
                 '\n'
-                '${lp}    /* Read the string variable from the buffer */\n'
-                '${lp}    qmi_utils_read_string_from_buffer (\n'
-                '${lp}        &${buffer_name},\n'
-                '${lp}        &${buffer_len},\n'
-                '${lp}        ${length_prefix_size},\n'
-                '${lp}        ${max_size},\n'
-                '${lp}        &tmp);\n'
-                '\n'
-                '${lp}    g_string_append_printf (${printable}, "%s", tmp);\n'
+                '${lp}    if (!qmi_message_tlv_read_string (message, init_offset, &offset, ${n_size_prefix_bytes}, ${max_size}, &tmp, &error))\n'
+                '${lp}        goto out;\n'
+                '${lp}    g_string_append (printable, tmp);\n'
                 '${lp}    g_free (tmp);\n'
                 '${lp}}\n')
 
@@ -224,11 +169,17 @@ class VariableString(Variable):
     """
     Variable declaration
     """
-    def build_variable_declaration(self, line_prefix, variable_name):
+    def build_variable_declaration(self, public, line_prefix, variable_name):
         translations = { 'lp'   : line_prefix,
                          'name' : variable_name }
 
-        if self.is_fixed_size:
+        if public:
+            # Fixed sized strings given in public structs are given as pointers,
+            # instead of as fixed-sized arrays directly in the struct.
+            template = (
+                '${lp}gchar *${name};\n')
+            self.is_public = True
+        elif self.is_fixed_size:
             translations['fixed_size_plus_one'] = int(self.fixed_size) + 1
             template = (
                 '${lp}gchar ${name}[${fixed_size_plus_one}];\n')
@@ -379,7 +330,7 @@ class VariableString(Variable):
     """
     def build_dispose(self, line_prefix, variable_name):
         # Fixed-size strings don't need dispose
-        if self.is_fixed_size:
+        if self.is_fixed_size and not self.public:
             return ''
 
         translations = { 'lp'            : line_prefix,
@@ -388,3 +339,14 @@ class VariableString(Variable):
         template = (
             '${lp}g_free (${variable_name});\n')
         return string.Template(template).substitute(translations)
+
+
+    """
+    Flag as being public
+    """
+    def flag_public(self):
+        # Call the parent method
+        Variable.flag_public(self)
+        # Fixed-sized strings will need dispose if they are in the public header
+        if self.is_fixed_size:
+            self.needs_dispose = True
