@@ -24,7 +24,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2012-2014 Aleksander Morgado <aleksander@aleksander.es>
+ * Copyright (C) 2012-2015 Aleksander Morgado <aleksander@aleksander.es>
  */
 
 #include <glib.h>
@@ -50,6 +50,7 @@
 #include "qmi-uim.h"
 #include "qmi-oma.h"
 #include "qmi-wda.h"
+#include "qmi-voice.h"
 
 /**
  * SECTION:qmi-message
@@ -2161,6 +2162,9 @@ qmi_message_get_printable (QmiMessage *self,
     case QMI_SERVICE_WDA:
         contents = __qmi_message_wda_get_printable (self, line_prefix);
         break;
+    case QMI_SERVICE_VOICE:
+        contents = __qmi_message_voice_get_printable (self, line_prefix);
+        break;
     default:
         break;
     }
